@@ -57,11 +57,12 @@ class Filesystem(object):
 
         gitdir: pathname to the bare repository or the .git directory
         """
-        scanroot = os.path.join(gitdir, "refs")
-        log.debug("Starting git scanning at %s", scanroot)
-        for root, dirs, files in os.walk(scanroot):
-            for f in files:
-                self.scan_file(os.path.join(root, f))
+        for subdir in ("refs", "objects"):
+            scanroot = os.path.join(gitdir, subdir)
+            log.debug("git scanning at %s", scanroot)
+            for root, dirs, files in os.walk(scanroot):
+                for f in files:
+                    self.scan_file(os.path.join(root, f))
 
     def scan_svn_repo(self, svnroot):
         """
@@ -88,7 +89,7 @@ class Filesystem(object):
         """
         Generate (ident, begin, end) contributions
         """
-        for uid, stats in self.contribs.iteritems():
+        for uid, stats in self.contribs.items():
             try:
                 pw = pwd.getpwuid(uid)
                 ident = Identifier("login", pw.pw_name)
