@@ -23,6 +23,8 @@ from collections import defaultdict
 from .types import *
 import sys
 import os
+import io
+import six
 import json
 
 __all__ = ["Submission"]
@@ -93,7 +95,7 @@ class Submission(object):
         if self.name != submission.name:
             raise ValueError("Merging submission for two different sources: {}!={}".format(
                 self.name, submission.name))
-        for ident, contribs in submission.entries.iteritems():
+        for ident, contribs in submission.entries.items():
             old = self.entries.get(ident, None)
             if old is None:
                 self.entries[ident] = dict(contribs)
@@ -104,10 +106,10 @@ class Submission(object):
         """
         Generate DC records for serialization
         """
-        for ident, contributions in self.entries.iteritems():
+        for ident, contributions in self.entries.items():
             yield {
                 "id": (ident.to_json(),),
-                "contributions": [ c.to_json() for c in contributions.itervalues() ],
+                "contributions": [ c.to_json() for c in contributions.values() ],
             }
 
     def to_json(self, file=None, indent=None):
@@ -127,8 +129,8 @@ class Submission(object):
         """
         Make a compact dump of this source to the given file
         """
-        for ident, contributions in self.entries.iteritems():
-            for ctype, c in sorted(contributions.iteritems()):
+        for ident, contributions in self.entries.items():
+            for ctype, c in sorted(contributions.items()):
                 if ident.desc:
                     lead = "{}:{} <{}>".format(ident.type, ident.desc, ident.id)
                 else:
@@ -152,11 +154,10 @@ class Submission(object):
         # Yuck! Python's stdlib cannot do file uploads :'(
         # We need to introduce an external dependency for it
         import requests
-        import urlparse
-        from cStringIO import StringIO
+        from six.moves.urllib.parse import urljoin
 
         # Build the POST request to contributors.debian.org
-        url = urlparse.urljoin(self.baseurl, '/contributors/post')
+        url = urljoin(self.baseurl, '/contributors/post')
         #print("Posting to '{}'...".format(url))
 
         # Prepare the file to post
@@ -168,12 +169,12 @@ class Submission(object):
             import gzip
             compress_type = "gzip"
             def compress(data):
-                out = StringIO()
+                out = io.BytesIO()
                 with gzip.GzipFile(mode="wb", fileobj=out) as fd:
                     fd.write(data)
                 return out.getvalue()
 
-        file_data = StringIO(compress(self.to_json()))
+        file_data = io.BytesIO(compress(self.to_json().encode("utf-8")))
         files = {
             "data": file_data
         }
@@ -194,21 +195,29 @@ class Submission(object):
         # certificates (like *.debian.org machines), use them
         if os.path.exists("/etc/ssl/ca-debian/ca-certificates.crt"):
             args["verify"] = "/etc/ssl/ca-debian/ca-certificates.crt"
+        else:
+            args["verify"] = True
         try:
             res = requests.post(url, **args)
-        except requests.ConnectionError, e:
+            res.raise_for_status()
+        except requests.ConnectionError as e:
             return False, {
                 "code": None,
                 "errors": [
-                    "Connection error: " + unicode(e)
+                    "Connection error: " + str(e)
                 ]
             }
-        except requests.HTTPError, e:
+        except requests.HTTPError as e:
+            try:
+                parsed = json.loads(res.text)
+                errors = parsed["errors"]
+            except:
+                errors = []
             return False, {
                 "code": None,
                 "errors": [
-                    "Server's HTTP response was malformed: " + unicode(e)
-                ]
+                    "Server responded with an HTTP error: " + six.text_type(e)
+                ] + errors
             }
 
         # Whether the POST was successful or not, the response body contains
@@ -228,7 +237,7 @@ class Submission(object):
         data: the JSON data, either in a string, in a file, or as a parsed data
               structure
         """
-        if isinstance(data, basestring):
+        if isinstance(data, six.string_types):
             data = json.loads(data)
         elif hasattr(data, "read"):
             data = json.load(data)
@@ -253,7 +262,7 @@ def merge_contrib_dicts(d1, d2):
     res = {}
 
     # Add elements from d1, merging them with d2 if they also exist in d2
-    for ctype, c1 in d1.iteritems():
+    for ctype, c1 in d1.items():
         c2 = d2.get(ctype, None)
         if c2 is None:
             res[ctype] = c1
@@ -261,7 +270,7 @@ def merge_contrib_dicts(d1, d2):
             res[ctype] = Contribution.merged(c1, c2)
 
     # Add the elements that only exist in d2
-    for ctype, c2 in d2.iteritems():
+    for ctype, c2 in d2.items():
         res.setdefault(ctype, c2)
 
     return res
