@@ -194,7 +194,7 @@ class Contribution(object):
         elif first.end is None:
             end = second.end
         else:
-            end = min(first.end, second.end)
+            end = max(first.end, second.end)
 
         if first.url is None:
             url = second.url
